function table = syndtable(par)
%SYNDTABLE Produce syndrome decoding table.
%   T = SYNDTABLE(PAR) returns a binary matrix that represents the first column of
%   a standard array of a linear code having a parity-check matrix H.  T
%   consists of coset leaders ordered sequentially by the associated syndromes
%   such that the first row contains an error pattern with a syndrome equal to
%   0, and the last row contains an error pattern with a syndrome equal to
%   2^(number of rows in H)-1.
%
% (c) 2017 Jan Prikryl
%
% Inspired by an alternative approach to syndrome table calulation as
% discussed in http://math.stackexchange.com/questions/1505517/constructing-syndrome-decoding-table

if isempty(par)
    error('syndtable:ParityCheckEmpty', ...
        'The parity check matrix cannot be empty.');
end

[nk,n]=size(par);  % nk denotes n-k
if nk >= n || ~isequal(size(size(par)),[1 2])
    error('syndtable:MoreRowsThanColumns', ...
        'There must be more columns than rows in the parity check matrix.');
end

if ~isnumeric(par) || ...
        max(max(par < 0)) || max(max(par)) > 1 || ...
        (max(max(floor(par) ~= par)))
    error('syndtable:NonBinary', ...
        'The parity check matrix must contain only binary numbers.');
end

% The number of syndromes is 2^nk. This is also the number of coset
% leaders.
num_syndromes = 2^nk;
syndrome_tab = zeros(num_syndromes, nk);
cosetlead_tab = zeros(num_syndromes, n);

% Generate all syndromes that may occur
for i=0:(2^nk-1)
    syndrome_tab(i+1,:) = dec2binvec(i, nk);
end

% Check to see if each computed syndrome is equal to some column in the
% parity check matrix:
% * if it does, put a 1 in that column of the coset leader vector
% * if it does not, make a linear combination of the columns if the 
%   syndrome does not correspond to a single column.
for i=0:(2^nk-1)
    syndrome = syndrome_tab(i+1,:);
    [~,idx] = ismember(syndrome, par', 'rows');
    % Do we have a direct match?
    if idx > 0
        % Create a coset leader
        cosetlead_tab(i+1,idx) = 1;
    end
end

% There are probably still elements that have not been processed. These
% elements are marked by a row full of zeros at the position of a coset
% leader.
for syndrome_dec = (find(~sum(cosetlead_tab,2)')-1)
    if syndrome_dec>0
        % Makes sense only for non-zero syndromes
        syndrome = syndrome_tab(syndrome_dec+1,:);
        parent_powers = find(syndrome) - 1;
        parent_idxs = 2.^parent_powers + 1;
        cosets = cosetlead_tab(parent_idxs,:);
        % Sanity check: No zero row allowed
        zero_rows = find(~sum(cosets,2));
        if ~empty(zero_rows)
            error('syndtable:CosetLeaderZero', ...
                'A primitive coset leader may not be zero.');
        end
        coset_lead = sum(cosets);
    end
end


table = cosetlead_tab(2:end,:);

end

